@@ -421,12 +421,32 @@ class AdvancedHTMLParser(HTMLParser):
 
     @property
     def body(self):
+        '''
+            body - Get the body element
+
+            @return <AdvancedTag> - The body tag, or None if no body tag present
+        '''
         return self.getFirstElementCustomFilter(lambda em : em.tagName == 'body')
 
     @property
     def head(self):
+        '''
+            head - Get the head element
+
+            @return <AdvancedTag> - The head tag, or None if no head tag present
+        '''
         return self.getFirstElementCustomFilter(lambda em : em.tagName == 'head')
 
+    @property
+    def forms(self):
+        '''
+            forms - Return all forms associated with this document
+
+            @return <TagCollection> - All "form" elements
+        '''
+
+        return self.getElementsByTagName('form')
+
     def contains(self, em):
         '''
             Checks if #em is found anywhere within this element tree
