@@ -12,7 +12,28 @@ from .SpecialAttributes import SpecialAttributesDict, StyleAttribute, AttributeN
 
 from .utils import escapeQuotes
 
-__all__ = ('AdvancedTag', 'uniqueTags', 'TagCollection', 'FilterableTagCollection', 'toggleAttributesDOM')
+__all__ = ('AdvancedTag', 'uniqueTags', 'TagCollection', 'FilterableTagCollection', 'toggleAttributesDOM', 'isTextNode', 'isTagNode')
+
+
+def isTextNode(node):
+    '''
+        isTextNode - Test if given node is a text node (Not a tag)
+
+        @param node - Node to test
+
+        @return bool
+    '''
+    return not issubclass(node.__class__, AdvancedTag)
+
+def isTagNode(node):
+    '''
+        isTagNode - Test if given node is a tag node (AdvancedTag)
+
+        @param node - Node to test
+
+        @return bool
+    '''
+    return issubclass(node.__class__, AdvancedTag)
 
 
 def uniqueTags(tagList):
@@ -194,22 +215,64 @@ class AdvancedTag(object):
 
     def removeText(self, text):
         '''
-            removeText - Removes some inner text
+            removeText - Removes the first occurace of given text in a text node (i.e. not part of a tag)
+
+            @param text <str> - text to remove
+
+            @return text <str/None> - The text in that block (text node) after remove, or None if not found
+
+            NOTE: To remove a node, @see removeChild
+            NOTE: To remove a block (maybe a node, maybe text), @see removeBlock
+            NOTE: To remove ALL occuraces of text, @see removeTextAll
         '''
-        newBlocks = []
-        for block in self.blocks:
-            if isinstance(block, AdvancedTag):
-                newBlocks.append(block)
+        removedBlock = None
+
+        blocks = self.blocks
+        for i in range(len(blocks)):
+            block = blocks[i]
+            if issubclass(block.__class__, AdvancedTag):
                 continue
+
             if text in block:
-                block = block.replace(text, '')
-            if block:
-                newBlocks.append(block)
+                removedBlock = block[:]
+                self.blocks[i] = block.replace(text, '')
+                break # remove should only remove FIRST occurace, per other methods
 
-        del block # pyflakes warning
-        self.blocks = newBlocks
+        self.text = ''.join([thisBlock for thisBlock in blocks if not issubclass(thisBlock.__class__, AdvancedTag)])
+
+        return removedBlock
+
+
+    def removeTextAll(self, text):
+        '''
+            removeTextAll - Removes ALL occuraces of given text in a text node (i.e. not part of a tag)
+
+            @param text <str> - text to remove
+
+            @return list <str> - All text node containing #text BEFORE the text was removed.
+                Empty list if no text removed
+
+            NOTE: To remove a node, @see removeChild
+            NOTE: To remove a block (maybe a node, maybe text), @see removeBlock
+            NOTE: To remove a single occurace of text, @see removeText
+        '''
+        removedBlocks = []
+
+        blocks = self.blocks
+        for i in range(len(blocks)):
+            block = blocks[i]
+            if issubclass(block.__class__, AdvancedTag):
+                continue
+
+            if text in block:
+                removedBlocks.append( block[:] )
+                self.blocks[i] = block.replace(text, '')
+
+
+        self.text = ''.join([thisBlock for thisBlock in blocks if not issubclass(thisBlock.__class__, AdvancedTag)])
+
+        return removedBlocks
 
-        self.text = ''.join([block for block in self.blocks if not isinstance(block, AdvancedTag)])
 
     def remove(self):
         '''
@@ -227,6 +290,50 @@ class AdvancedTag(object):
             return True
         return False
 
+    def removeBlock(self, block):
+        '''
+            removeBlock - Removes a block (the first occurance) from the direct children of this node.
+
+            @param block <str/AdvancedTag> - An AdvancedTag for a tag node, else a string for a text node
+
+            @return The removed block, or None if None removed.
+
+            @see removeChild
+            @see removeText
+
+            For multiple, @see removeBlocks
+        '''
+        if issubclass(block.__class__, AdvancedTag):
+            return self.removeChild(block)
+        else:
+            try:
+                return self.blocks.pop(block)
+            except:
+                return None
+
+
+    def removeBlocks(self, blocks):
+        '''
+            removeBlock - Removes a list of blocks (the first occurance of each) from the direct children of this node.
+
+            @param blocks  list<str/AdvancedTag> - List of AdvancedTags for tag nodes, else strings for text nodes
+
+            @return The removed blocks in each slot, or None if None removed.
+
+            @see removeChild
+            @see removeText
+
+            For multiple, @see removeBlocks
+        '''
+        ret = []
+        for block in blocks:
+            if issubclass(block.__class__, AdvancedTag):
+                ret.append( self.removeChild(block) )
+            else:
+                ret.append( self.removeBlock(block) )
+
+        return ret
+
     def appendChild(self, child):
         '''
             appendChild - Append a child to this element.
@@ -245,6 +352,45 @@ class AdvancedTag(object):
 
     appendNode = appendChild
 
+    def appendBlock(self, block):
+        '''
+            appendBlock - Append a block to this element. A block can be a string (text node), or an AdvancedTag (tag node)
+
+            @param <str/AdvancedTag> - block to add
+
+            @return - #block
+
+            NOTE: To add multiple blocks, @see appendBlocks
+                  If you know the type, use either @see appendChild for tags or @see appendText for text
+        '''
+        if isinstance(block, AdvancedTag):
+            self.appendNode(block)
+        else:
+            self.appendText(block)
+        
+        return block
+
+
+    def appendBlocks(self, blocks):
+        '''
+            appendBlocks - Append blocks to this element. A block can be a string (text node), or an AdvancedTag (tag node)
+
+            @param blocks list<str/AdvancedTag> - A list, in order to append, of blocks to add.
+
+            @return - #blocks
+
+            NOTE: To add a single block, @see appendBlock
+                  If you know the type, use either @see appendChild for tags or @see appendText for text
+        '''
+        for block in blocks:
+            if isinstance(block, AdvancedTag):
+                self.appendNode(block)
+            else:
+                self.appendText(block)
+
+        return blocks
+
+
     def appendInnerHTML(self, html):
         '''
             appendInnerHTML - Appends nodes from arbitrary HTML as if doing element.innerHTML += 'someHTML' in javascript.
@@ -265,18 +411,21 @@ class AdvancedTag(object):
 
         blocks = AdvancedHTMLParser.createBlocksFromHTML(html, encoding)
 
-        for block in blocks:
-            if isinstance(block, AdvancedTag):
-                self.appendNode(block)
-            else:
-                self.appendText(block)
+        self.appendBlocks(blocks)
+
 
     def removeChild(self, child):
         '''
-            removeChild - Remove a child, if present.
+            removeChild - Remove a child tag, if present.
+
+                @param child <AdvancedTag> - The child to remove
 
-                @param child - The child to remove
                 @return - The child [with parentNode cleared] if removed, otherwise None.
+
+                NOTE: This removes a tag. If removing a text block, use #removeText function.
+                  If you need to remove an arbitrary block (text or AdvancedTag), @see removeBlock
+
+                Removing multiple children? @see removeChildren
         '''
         try:
             self.children.remove(child)
@@ -291,45 +440,104 @@ class AdvancedTag(object):
 
     removeNode = removeChild
 
+    def removeChildren(self, children):
+        '''
+            removeChildren - Remove multiple child AdvancedTags.
+
+            @see removeChild
+
+            @return list<AdvancedTag/None> - A list of all tags removed in same order as passed.
+                Item is "None" if it was not attached to this node, and thus was not removed.
+        '''
+        ret = []
+
+        for child in children:
+            ret.append( self.removeChild(child) )
+
+        return ret
+            
+
+    def removeBlock(self, block):
+        '''
+            removeBlock - Removes a single block (text node or AdvancedTag) which is a child of this object.
+
+            @param block <str/AdvancedTag> - The block (text node or AdvancedTag) to remove.
+            
+            @return Returns the removed block if one was removed, or None if requested block is not a child of this node.
+
+            NOTE: If you know you are going to remove an AdvancedTag, @see removeChild
+                  If you know you are going to remove a text node,    @see removeText
+
+            If removing multiple blocks, @see removeBlocks
+        '''
+        if issubclass(block.__class__, AdvancedTag):
+            return self.removeChild(block)
+        else:
+            return self.removeText(block)
+
+            
+
     def insertBefore(self, child, beforeChild):
         '''
-            insertBefore - Inserts a child before @beforeChild
+            insertBefore - Inserts a child before #beforeChild
 
-            child - Child to insert
-            beforeChild - Child  to insert before. if None, will  be appended
+
+                @param child <AdvancedTag/str> - Child block to insert
+
+                @param beforeChild <AdvancedTag/str> - Child block to insert before. if None, will  be appended
+
+            @return - The added child. Note, if it is a text block (str), the return isl NOT be linked by reference.
 
         '''
         if beforeChild is None:
-            return self.appendChild(child)
+            return self.appendBlock(child)
+
+        isChildTag = isTagNode(child)
 
         try:
-            childrenIdx = self.children.index(beforeChild)
             blocksIdx =  self.blocks.index(beforeChild)
-            self.children = self.children[:childrenIdx] + [child] + self.children[childrenIdx:]
-            self.blocks = self.blocks[:blocksIdx] + [child] + self.blocks[blocksIdx:]
+            if isChildTag:
+                childrenIdx = self.children.index(beforeChild)
         except ValueError:
             raise ValueError('Provided "beforeChild" is not a child of element, cannot insert.')
+        
+        self.blocks = self.blocks[:blocksIdx] + [child] + self.blocks[blocksIdx:]
+        if isChildTag: 
+            self.children = self.children[:childrenIdx] + [child] + self.children[childrenIdx:]
+        
+        return child
 
     def insertAfter(self, child, afterChild):
         '''
-            insertAfter - Inserts a child after @afterChild
+            insertAfter - Inserts a child after #afterChild
 
-            child - Child to insert
-            afterChild - Child  to insert after. if None, will  be appended
 
+                @param child <AdvancedTag/str> - Child block to insert
+
+                @param afterChild <AdvancedTag/str> - Child block to insert after. if None, will  be appended
+
+            @return - The added child. Note, if it is a text block (str), the return isl NOT be linked by reference.
         '''
         if afterChild is None:
-            return self.appendChild(child)
+            return self.appendBlock(child)
+
+        isChildTag = isTagNode(child)
 
         try:
-            childrenIdx = self.children.index(afterChild)
             blocksIdx =  self.blocks.index(afterChild)
-            self.children = self.children[:childrenIdx+1] + [child] + self.children[childrenIdx+1:]
-            self.blocks = self.blocks[:blocksIdx+1] + [child] + self.blocks[blocksIdx+1:]
+            if isChildTag:
+                childrenIdx = self.children.index(afterChild)
         except ValueError:
             raise ValueError('Provided "afterChild" is not a child of element, cannot insert.')
 
 
+        self.blocks = self.blocks[:blocksIdx+1] + [child] + self.blocks[blocksIdx+1:]
+        if isChildTag:
+            self.children = self.children[:childrenIdx+1] + [child] + self.children[childrenIdx+1:]
+
+        return child
+
+
     # Maybe we want to do a more full implementation of the Node stuff.... but I don't think anyone really
     #   uses this stuff
     @property
@@ -573,9 +781,36 @@ class AdvancedTag(object):
             childNodes - returns immediate child nodes as a TagCollection
 
             @return - TagCollection of child nodes
+
+            NOTE: Unlike JS DOM, this returns ONLY tags, not text blocks.
+                   Changing this would be a fairly-major backwards-incompatible change,
+                   and will likely be made in a future version.
+
+                   For now, use @see childBlocks method to get both text AND tags
         '''
         return TagCollection(self.children)
 
+    @property
+    def childBlocks(self):
+        '''
+            childBlocks - Return immediate child blocks, both text and tags.
+
+            @return list<AdvancedTag/str> - List of blocks associated with this node
+
+            NOTE: This does what #childNodes does in JS DOM. Because for many years childNodes has returned
+              ONLY tags on AdvancedHTMLParser, it would be a major change to match. Likely will be made in a future
+              version.
+        '''
+        return self.blocks
+
+    def getChildBlocks(self):
+        '''
+            getChildBlocks - Gets the child blocks. 
+
+            @see childBlocks
+        '''
+        return self.childBlocks
+
     @property
     def childElementCount(self):
         '''
